@@ -35,7 +35,7 @@
 /* Some DJGPP builds define __unix__ although they don't support mmap().
  * Cygwin defines __unix__ and supports mmap(), but it does not work well.
  */
-#if !defined(__MSDOS__) && !defined(_WIN32) && (defined(unix) || defined(__unix__) || defined(__unix)) || (defined(__MACH__) && defined(__APPLE__))
+#if !defined(__MSDOS__) && !IS_WINDOWS && (defined(unix) || defined(__unix__) || defined(__unix)) || (defined(__MACH__) && defined(__APPLE__))
 #define HAVE_MMAP 1
 #include <sys/mman.h>
 #endif
@@ -113,14 +113,17 @@ static void usage(char *argv[], char *error)
 "where <image file name> points to an image of the contents of the SPI flash.\n"
 "In case the image is really in descriptor mode %s\n"
 "will pretty print some of the contained information.\n"
-"To also print the data stored in the descriptor strap you have to indicate\n"
+"To also print the data stored in the descriptor straps you have to indicate\n"
 "the chipset series with the '-c' parameter and one of the possible arguments:\n"
 "\t- \"ich8\",\n"
 "\t- \"ich9\",\n"
 "\t- \"ich10\",\n"
+"\t- \"silvermont\" for chipsets from Intel's Silvermont architecture (e.g. Bay Trail),\n"
 "\t- \"5\" or \"ibex\" for Intel's 5 series chipsets,\n"
 "\t- \"6\" or \"cougar\" for Intel's 6 series chipsets,\n"
 "\t- \"7\" or \"panther\" for Intel's 7 series chipsets.\n"
+"\t- \"8\" or \"lynx\" for Intel's 8 series chipsets.\n"
+"\t- \"9\" or \"wildcat\" for Intel's 9 series chipsets.\n"
 "If '-d' is specified some regions such as the BIOS image as seen by the CPU or\n"
 "the GbE blob that is required to initialize the GbE are also dumped to files.\n",
 	argv[0], argv[0]);
@@ -167,7 +170,7 @@ int main(int argc, char *argv[])
 		usage(argv, "Seeking to the end of the file failed");
 
 #ifdef HAVE_MMAP
-	buf = mmap(NULL, len, PROT_READ, MAP_PRIVATE, fd, 0);
+	buf = (uint32_t *)mmap(NULL, len, PROT_READ, MAP_PRIVATE, fd, 0);
 	if (buf == (void *) -1)
 #endif
 	{
@@ -177,7 +180,7 @@ int main(int argc, char *argv[])
 			usage(argv, "Could not allocate memory");
 		lseek(fd, 0, SEEK_SET);
 		if (len != read(fd, buf, len))
-			usage(argv, "Seeking to the end of the file failed");
+			usage(argv, "Reading the descriptor image file failed");
 	}
 	printf("The flash image has a size of %d [0x%x] bytes.\n", len, len);
 	close(fd);
@@ -198,6 +201,14 @@ int main(int argc, char *argv[])
 		else if ((strcmp(csn, "7") == 0) ||
 			 (strcmp(csn, "panther") == 0))
 			cs = CHIPSET_7_SERIES_PANTHER_POINT;
+		else if ((strcmp(csn, "8") == 0) ||
+			 (strcmp(csn, "lynx") == 0))
+			cs = CHIPSET_8_SERIES_LYNX_POINT;
+		else if ((strcmp(csn, "silvermont") == 0))
+			cs = CHIPSET_BAYTRAIL;
+		else if ((strcmp(csn, "9") == 0) ||
+			 (strcmp(csn, "wildcat") == 0))
+			cs = CHIPSET_9_SERIES_WILDCAT_POINT;
 	}
 
 	ret = read_ich_descriptors_from_dump(buf, len, &desc);
